#include "Hud.h"

#include "System.h"
#include "StringConverter.h"
#include "GameObject.h"
#include "Gui.h"
#include "Game.h"
#include "Vehicle.h"
#include "Camera.h"
#include "Weapon.h"

Hud::Hud(Gui* gui){
	this->gui = gui;
	this->overlay = Ogre::OverlayManager::getSingleton().getByName("Gui/Hud");
	this->status = Ogre::OverlayManager::getSingleton().getOverlayElement("Gui/Hud/Status");
	this->energyBar = Ogre::OverlayManager::getSingleton().getOverlayElement("Gui/Hud/Status/EnergyBar");
	this->healthBar = Ogre::OverlayManager::getSingleton().getOverlayElement("Gui/Hud/Status/HealthBar");

	for( int i=0; i<4; i++ ){
		string baseName = "Gui/Hud/Status/Weapon" + StringConverter::toString(i+1);

 		this->weaponInfos[i].panel = Ogre::OverlayManager::getSingleton().getOverlayElement( baseName );
		this->weaponInfos[i].icon = Ogre::OverlayManager::getSingleton().getOverlayElement( baseName + "/Icon" );
		this->weaponInfos[i].ammoIndicator = Ogre::OverlayManager::getSingleton().getOverlayElement( baseName + "/AmmoIndicator" );
		this->weaponInfos[i].heatIndicator = Ogre::OverlayManager::getSingleton().getOverlayElement( baseName + "/HeatIndicator" );
	}
}

Hud::~Hud(){

}

void Hud::show(){
	this->overlay->show();
}

void Hud::hide(){
	this->overlay->hide();
}
bool Hud::isVisible() const {
	return this->overlay->isVisible();
}

void Hud::update(){
	GameObject* camTarget = this->gui->getSystem()->getGame()->getCamera()->getTarget();

	if( camTarget != NULL && camTarget->getGameObjectType() == GameObject::GAME_OBJECT_TYPE_VEHICLE ){
		Vehicle* v = (Vehicle*)camTarget;

		// health and energy bars
		float energyRatio = v->getEnergy() / v->getPhysicalProperties().energyCapacity;
		float healthRatio = v->getHealth() / v->getPhysicalProperties().healthCapacity;

		this->energyBar->setWidth( energyRatio*0.169f );
		this->healthBar->setWidth( healthRatio*0.169f );

		// weapons
		for( int i=0; i<4; i++ ){ // must iterate over all slots
			Weapon* w = v->getWeapon(i);
			if( w != NULL ){
				if( !w->getGraphicalProperties().hudIcon.empty() ){
					this->weaponInfos[i].icon->setMaterialName( w->getGraphicalProperties().hudIcon );
					this->weaponInfos[i].icon->show();
				}else{
					this->weaponInfos[i].icon->hide();
				}

				const Weapon::physicalProperties_s& pp = w->getPhysicalProperties();
				if( pp.ammoCapacity > 0 ){ // do ammo
					float relAmmo = w->getAmmo() / pp.ammoCapacity;
					FColor c = FColor::WHITE;

					if( !w->isReady() ){ // darken if weapon is not ready
						c = c * 0.5f;
					}
					string colorString = StringConverter::toString(c.r) + " " + StringConverter::toString(c.g) + " " + StringConverter::toString(c.b);
					this->weaponInfos[i].ammoIndicator->setParameter( "colour", colorString );
					this->weaponInfos[i].ammoIndicator->setCaption( StringConverter::toString( w->getAmmo() ) );

					this->weaponInfos[i].ammoIndicator->show();
				}else{
					this->weaponInfos[i].ammoIndicator->hide();
				}

				if( pp.heatCapacity > 0 ){ // do heat
					float relHeat = w->getHeat() / pp.heatCapacity;
					this->weaponInfos[i].heatIndicator->setWidth( relHeat * 0.044f );
					if( w->isOverheated() ){
						this->weaponInfos[i].heatIndicator->setMaterialName( "gui/hud/status/weapons/overheated_bar" );
					}else{
						this->weaponInfos[i].heatIndicator->setMaterialName( "gui/hud/status/weapons/heat_bar" );
					}

					this->weaponInfos[i].heatIndicator->show();
				}else{
					this->weaponInfos[i].heatIndicator->hide();
				}

				this->weaponInfos[i].panel->show();
			}else{
				this->weaponInfos[i].panel->hide();
			}
		}

		this->status->show();
	}else{
		this->status->hide();
	}

}
